<?php
/**
 * Flexpage
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://opensource.org/licenses/gpl-3.0.html.
 *
 * @copyright Copyright (c) 2009 Moodlerooms Inc. (http://www.moodlerooms.com)
 * @license http://opensource.org/licenses/gpl-3.0.html GNU Public License
 * @package format_flexpage
 * @author Mark Nielsen
 */

/**
 * Methods for handling random module tasks
 *
 * @author Mark Nielsen
 * @package format_flexpage
 */
class course_format_flexpage_lib_mod {
    /**
     * Get options for adding activities
     *
     * @static
     * @param object|null $course Course object
     * @return array
     */
    public static function get_add_options($course = null) {
        global $CFG, $COURSE;

        require_once($CFG->dirroot.'/course/lib.php');

        if (is_null($course)) {
            $course = $COURSE;
        }
        $urlbase       = "/course/mod.php?id=$course->id&section=0&sesskey=".sesskey().'&add=';
        $stractivities = get_string('activities');
        $strresources  = get_string('resources');
        $options       = array(
            $stractivities => array(),
            $strresources  => array(),
        );

        get_all_mods($course->id, $mods, $modnames, $modnamesplural, $modnamesused);

        foreach($modnames as $modname => $modnamestr) {
            if (!course_allowed_module($course, $modname)) {
                continue;
            }

            try {
                if ($types = self::callback($modname, 'get_types')) {
                    $menu = array();
                    $atype = null;
                    $groupname = null;
                    foreach($types as $type) {
                        if ($type->typestr === '--') {
                            continue;
                        }
                        if (strpos($type->typestr, '--') === 0) {
                            $groupname = str_replace('--', '', $type->typestr);
                            continue;
                        }
                        $type->type = str_replace('&amp;', '&', $type->type);
                        if ($type->modclass == MOD_CLASS_RESOURCE) {
                            $atype = MOD_CLASS_RESOURCE;
                        }
                        $menu[$urlbase.$type->type] = array(
                            'module' => $modname,
                            'label'  => $type->typestr
                        );
                    }
                    if (is_null($groupname)) {
                        if ($atype == MOD_CLASS_RESOURCE) {
                            $groupname = $strresources;
                        } else {
                            $groupname = $stractivities;
                        }
                    }
                    if (empty($options[$groupname])) {
                        $options[$groupname] = $menu;
                    } else {
                        $options[$groupname] = array_merge($options[$groupname], $menu);
                    }
                }
            } catch (coding_exception $e) {
                $archetype = plugin_supports('mod', $modname, FEATURE_MOD_ARCHETYPE, MOD_ARCHETYPE_OTHER);
                if ($archetype == MOD_ARCHETYPE_RESOURCE) {
                    $groupname = $strresources;
                } else {
                    $groupname = $stractivities;
                }
                $options[$groupname][$urlbase.$modname] = array(
                    'module' => $modname,
                    'label'  => $modnamestr,
                );
            }
        }
        foreach ($options as $groupname => $activities) {
            uasort($activities, 'course_format_flexpage_lib_mod::sort_options');
            $options[$groupname] = $activities;
        }
        return $options;
    }

    /**
     * Get options for adding existing activities
     *
     * @static
     * @param object|null $course Course object
     * @return array
     */
    public static function get_existing_options($course = null) {
        global $COURSE;

        if (is_null($course)) {
            $course = $COURSE;
        }
        $modinfo = get_fast_modinfo($course);
        $options = array();
        foreach ($modinfo->get_instances() as $module => $instances) {
            $group = array();
            foreach ($instances as $instance) {
                $group[$instance->id] = array(
                    'module' => $instance->modname,
                    'label' => $instance->name,
                );
            }
            uasort($group, 'course_format_flexpage_lib_mod::sort_options');

            $options[get_string('modulenameplural', $module)] = $group;
        }
        ksort($options);

        return $options;
    }

    /**
     * Sorting method, used internally
     *
     * @static
     * @param  $a
     * @param  $b
     * @return int
     */
    public static function sort_options($a, $b) {
        return strnatcasecmp($a['label'], $b['label']);
    }

    /**
     * Perform a module callback
     *
     * @param string $module Module dir name
     * @param string $function The function name to call
     * @param array $arguments Args to pass to the function
     * @throws coding_exception
     * @return mixed
     */
    public static function callback($module, $function, $arguments = array()) {
        global $CFG;

        $lib = "$CFG->dirroot/mod/$module/lib.php";

        if (!file_exists($lib)) {
            throw new coding_exception('Module lib file does not exist');
        }
        @require_once($lib);

        $function = "{$module}_$function";

        if (!function_exists($function)) {
            throw new coding_exception("Function does not exist: $function");
        }
        return call_user_func_array($function, $arguments);
    }
}